package org.jbpm.task.test;

import static junit.framework.Assert.*;
import java.util.UUID;
import java.util.concurrent.CountDownLatch;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SystemEventListenerFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.task.service.TaskClient;
import org.jbpm.task.service.hornetq.CommandBasedHornetQWSHumanTaskHandler;
import org.jbpm.task.service.hornetq.HornetQTaskClientConnector;
import org.jbpm.task.service.hornetq.HornetQTaskClientHandler;
import org.junit.Test;

import com.sample.CompleteProcessEventListener;

public class ServerIntegrationPostgresTxOnSessionWriteTest {

    private final static String BPMN2_FILE_NAME = "internal/largeObjectPostgresTaskTx.bpmn2";
    
    /**
     * This test requires a (human-task) server to be running on 127.0.0.1
     */
    @Test
    public void test() {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newClassPathResource(BPMN2_FILE_NAME), ResourceType.BPMN2);
        if (kbuilder.hasErrors()) {
            System.out.println(kbuilder.getErrors());
            fail(kbuilder.getErrors().toString());
        }
        
        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
        
        TaskClient taskClient = new TaskClient(new HornetQTaskClientConnector("HornetQConnector" + UUID.randomUUID(),
                new HornetQTaskClientHandler(SystemEventListenerFactory.getSystemEventListener())));
        taskClient.connect("localhost", 5446);
        
        CommandBasedHornetQWSHumanTaskHandler handler = new CommandBasedHornetQWSHumanTaskHandler(ksession);
        handler.setClient(taskClient);
        handler.connect();
        
        ksession.getWorkItemManager().registerWorkItemHandler("Human Task", handler);
        
        CountDownLatch latch = new CountDownLatch(1);
        
        CompleteProcessEventListener processEventListener = new CompleteProcessEventListener(latch);
        ksession.addEventListener(processEventListener);
        ksession.startProcess("test.testproc");
        
        try {
            latch.await();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        try {
            taskClient.disconnect();
            handler.dispose();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
